<?php
/**
 * The implementation of the URL enumerator for a whole application
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * URL enumerator for a whole application.
 *
 * All private members are strings, which are as replacement for a colon beginning
 * string.
 * Example: string ":projectName" will be replaced by a content of variable $_projectName
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class AppUrlEnum extends UrlEasyGet
{
 /** @var string */
 private $_projectName;

 /** @var string */
 private $_blogEntryName;

 /** @var int */
 private $_blogEntryId;

 /** @var string */
 private $_blogCategoryName;

 /** @var int */
 private $_blogCategoryId;

 /** @var int */
 private $_blogCommentId;

 /**
  * year for the archive articles identification
  * e.g. 2007
  * @var int
  */
 private $_blogArchiveYear;

 /**
  * month string for the archive articles identification
  * e.g. 03 for March
  * @var int
  */
 private $_blogArchiveMonth;

 /**
  * Singleton instance
  * @var AppUrlEnum
  */
 private static $_instance = null;
 
//==============================================================================
/**
 * Creates object, sets XML file with a URL map.
 * Singleton pattern implementation makes "new" unavailable.
 */
public function __construct()
{
 $config=Zend_Registry::get("conf");
 //constructor with XML file 
 parent::__construct($config->mainDir
  . DIRECTORY_SEPARATOR . "AppUrlMap.xml");
}

//==============================================================================
/**
 * Singleton pattern implementation makes "clone" unavailable
 * @return void
 */
private function __clone()
{}

//==============================================================================
/**
 * Singleton pattern implementation.
 * Only one instance for a whole system.
 * @return AppUrlEnum
 */
public static function getInstance()
{
 if (null === self::$_instance) {
  self::$_instance = new self();
 }
 return self::$_instance;
}

//==============================================================================
/**
 * @var string name
 */
public function setBlogEntryName($name)
{
 $this->_blogEntryName=$name;
}

//==============================================================================
/**
 * @var string name
 */
public function setProjectName($name)
{
 $this->_projectName=$name;
}

//==============================================================================
/**
 * @var int $id
 */
public function setBlogEntryId($id)
{
 $this->_blogEntryId=$id;
}

//==============================================================================
/**
 * @var int $year
 */
public function setBlogArchiveYear($year)
{
 $this->_blogArchiveYear=$year;
}

//==============================================================================
/**
 * @var string $month
 */
public function setBlogArchiveMonth($month)
{
 $this->_blogArchiveMonth=$month;
}

//==============================================================================
/**
 * @var string $name
 */
public function setBlogCategoryName($name)
{
 $this->_blogCategoryName=$name;
}

//==============================================================================
/**
 * Sets an id for a category
 * @var int $id 
 */
public function setBlogCategoryId($id)
{
 $this->_blogCategoryId=$id;
}

//==============================================================================
/**
 * Sets an id for a comment
 * @var int $id 
 */
public function setBlogCommentId($id)
{
 $this->_blogCommentId=$id;
}

//==============================================================================
/**
 * Main function, returns URL which is defined by urlName in the XML file.
 * @param string $urlName
 * @return string
 */
public function __get($urlName)
{
 //get URL defined by the map
 $url=parent::__get($urlName);

 //replace parameters if in the url some exists
 $this->_replaceParams($url);
 
 //add baseUrl and return
 return AppBaseUrl::getBaseUrl(empty($url)).$url;
}

//==============================================================================
/**
 * Replace parameters in the URL by member variable content. Parameters are in
 * a XML file strings beginning with colon (e.g :param).
 * @param stringRef $url url which will be worked up
 */
private function _replaceParams(&$url)
{
 /* NOTE for strpos, use always strong non equivalence !== not just !=, see
    manual for strpos
 */
 if (strpos($url,":blogEntryName") !== false ) {
  $url = str_replace(":blogEntryName",$this->_blogEntryName,$url);
 }
 if (strpos($url,":blogEntryId") !== false ) {
  $url = str_replace(":blogEntryId",$this->_blogEntryId,$url);
 }
 if (strpos($url,":projectName") !== false ) {
  $url = str_replace(":projectName",$this->_projectName,$url);
 }
 if (strpos($url,":blogCategoryName") !== false ) {
  $url = str_replace(":blogCategoryName",$this->_blogCategoryName,$url);
 }
 if (strpos($url,":blogCategoryId") !== false ) {
  $url = str_replace(":blogCategoryId",$this->_blogCategoryId,$url);
 }
 if (strpos($url,":blogCommentId") !== false ) {
  $url = str_replace(":blogCommentId",$this->_blogCommentId,$url);
 }
 if (strpos($url,":blogArchYear") !== false ) {
  $url = str_replace(":blogArchYear",$this->_blogArchiveYear,$url);
 }
 if (strpos($url,":blogArchMonth") !== false ) {
  $url = str_replace(":blogArchMonth",$this->_blogArchiveMonth,$url);
 }
}

#END CLASS
}
