<?php
/**
 * Implementation for the SiteTemplatePlugin class.
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Controller plugin for using a simple two step view or a sidewide template.
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class SiteTemplatePlugin extends Zend_Controller_Plugin_Abstract
{
 //constants for main menu items
 const MAINMENU_ABOUT="about";

 /** current main menu item identifier, one of MAINMENU constants
  * @var string
  */
 private static $_mainMenu;

//==============================================================================
/**
 * Setter for the $_mainMenu
 * @param string $menuId menu identifier
 * @throws InvalidArgumentException if $menuId isn't one of MAINMENU constants
 */
public static function setMainMenu($menuId)
{
 switch ($menuId) {
  case self::MAINMENU_ABOUT:
   self::$_mainMenu=self::MAINMENU_ABOUT;
   break;
  default:
   throw new InvalidArgumentException("unknow main menu identifier: $menuId");
   break;
 }
}

//==============================================================================
/**
 * Plugin hook before front controller begins evaluating the request.
 * Sets language and locale if necessary parameters are presented. 
 */ 
public function routeStartup(Zend_Controller_Request_Abstract $request)
{
 $app=HomeWebApp::getInstance();
 
 if ($lang=$request->getParam('lang')) {
  $app->setLang($request->getParam('lang'));
  $request->setParam('lang',null);
 }
 
 //set language in viewrenderer views
 $viewRenderer=Zend_Controller_Action_HelperBroker::getStaticHelper('viewRenderer');
 //set language
 $viewRenderer->view->currentLang=$app->getLocale()->getLanguage();
 
 //set host part for the base application URL
 AppBaseUrl::setHost($request->getServer("SERVER_NAME"));
}

//==============================================================================
/**
 * Plugin hook after Zend_Controller_Front exits a whole dispatch loop.
 * 
 * We inject $response body to the sitewide template as the $content view variable
 * All view variables are persited because view is grabbed from the viewRenderer
 * plugin.
 *
 * We can affect the entire layout of the page, and also chain together several
 * actions to build the final page.
 */
public function dispatchLoopShutdown()
{
 $config = Zend_Registry::get("conf");

 //assume that we've already determined the request is ajax or image or whatever
 //in some action handler
 
 $request  = $this->getRequest();
 $response = $this->getResponse();
 $viewRenderer = Zend_Controller_Action_HelperBroker::getExistingHelper('viewRenderer');
 $view = $viewRenderer->view;

 //ajax response, no common template, just return
 if ($request->getParam('isAjax')) {
  return;
 }

 //rss/atom feed response, just return
 if ($request->getParam('isFeed')) {
  return;
 }

 //HTML response with sitewide template
 
 //content is response body, which is set in the action handler
 $view->content = $response->getBody();
 $view->addScriptPath($config->defaultViewDir.DIRECTORY_SEPARATOR."scripts");
 //right block
 if (empty($view->rightBlock)) $view->rightBlock=$view->render("links.phtml");

 //there isn't necessary use scripth path for the template, because a view path is defined in
 //the viewRenderer as moduledirs/views via setViewBasePathSpec()
 $response->setBody($view->render("mainTemplate.phtml"));
 $response->setHeader("Cache-Control","max-age=7200,must-revalidate",true);
}

#END CLASS
}
