<?php
/**
 * Base url getter that return the base url for a whole application
 *
 * @package home page
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Class that returns the base url for the current request.
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class AppBaseUrl
{
 /**
  * simple cache for the base url
  * @var string
  */
 private static $_baseUrl=null;

 /**
  * A host URI part for the application.
  * e.g. www.example.com
  * @var string
  */
 private static $_hostPart;

//==============================================================================
/**
 * Sets host part e.g. www.example.com
 * @param string $hostName
 */
public static function setHost($hostName)
{
 self::$_hostPart=$hostName;
}

//==============================================================================
/**
 * Gets URI - http://{@link $_hostPart}
 * @throws Exception when $_hostPart is empty.
 */
public static function getUri()
{
 if (empty(self::$_hostPart)) throw new Exception("host URI part isn't set");
 return "http://" . self::$_hostPart;
}

//==============================================================================
/**
 * Get the base url for the current request.
 * @param $withHostPart if is true, return whole URI, e.g.
 * http://www.example.com/baseUrl, if is false, return base URL only e.g. /baseUrl
 * @return string
 * @throws Exception when $_hostPart is empty and $withHostPart is true, which bubbles
 * from {@link $_getUri()}.
 */
public static function getBaseUrl($withHostPart=false)
{
 if (self::$_baseUrl == null)
  self::$_baseUrl = Zend_Controller_Front::getInstance()->getRequest()->getBaseUrl();

 if ($withHostPart) {
  if (empty(self::$_hostPart)) throw new Exception("host URI part isn't set");
  return self::getUri().self::$_baseUrl;
 } else {
  return self::$_baseUrl;
 }
}

#END CLASS
}
