<?php
/**
 * Action controller, which is responsible for a whole blog administration 
 * interface.
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Action controller, which is responsible for a whole blog administration 
 *
 * It was written for a simplycity as one controller,  maybe a better approach is
 * the using blog admin module or separated controllers for articles, categories
 * etc. Maybe in a future.   
 * 
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */ 
class Blog_AdminController extends Zend_Controller_Action
{
//==============================================================================
/**
 * The action controller initialization. It makes basic authorization also.
 */ 
public function init()
{
 $config = Zend_Registry::get("conf");
 
 //simple authorization code  
 //create Zend_Auth objects
 $authConfig = array(
    'accept_schemes' => 'digest',
    'realm'          => 'Admin Access',
    'digest_domains' => AppUrlEnum::getInstance()->adminIndex,
    'nonce_timeout'  => 3600,
 );
 $adapter = new Zend_Auth_Adapter_Http($authConfig);
 $basicResolver = new Zend_Auth_Adapter_Http_Resolver_File();
 $basicResolver->setFile($config->mainDir.DIRECTORY_SEPARATOR.".htpasswd");
 $adapter->setDigestResolver($basicResolver);

 $adapter->setRequest( $this->getRequest() );
 $adapter->setResponse( $this->getResponse() );
  
 $result = $adapter->authenticate();
 
 if (!$result->isValid()) {
  //view auth dialog and exit
  $adapter->getResponse()->sendResponse();
  exit;
 }
 
 //prepare helper path and right block as admin menu  
 $this->view->addHelperPath($config->defaultViewDir.DIRECTORY_SEPARATOR."helpers");
 $this->view->rightBlock=$this->view->render("adminMenu.phtml");
}

//==============================================================================
/**
 * Simple administration index.
 */ 
public function indexAction()
{
}

//==============================================================================
/**
 * List for last set of articles with pagination.
 */ 
public function entryListAction()
{
 $article = new Article();
 $itemsCount=$article->getCount();
 $this->view->articleCount=$itemsCount;
 
 if ($itemsCount) {
  //get current page number, default is 1
  $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;
  //page navigator
  $pgNav=new PageNav($pageNo,$itemsCount);
  $pgNav->MainUrl=AppUrlEnum::getInstance()->blogEntryList;

  //articles - current page
  $this->view->articles=$article->getAll($pageNo,$pgNav->getPerPageItems());
  
  //add page navigator XHTML code for a view
  $this->view->pageNav=$pgNav->getNavXhtmlContent();
 }

 if ($this->_helper->flashMessenger->hasMessages()) {
  $this->view->messages=$this->_helper->flashMessenger->getMessages();
 }
 
}

//==============================================================================
/**
 * Search for a article entry - view for a search form
 */ 
public function searchEntryAction()
{
}

//==============================================================================
/**
 * Action, which is aim for a search article entry dialog.
 * When input data are invalid, forwards to the {@link searchEntryAction()}. 
 */ 
public function searchEntryResultAction()
{
 if ($this->_hasParam("title")) {
  $data=$this->getRequest()->getPost();
  $filters = array(
    'title'   => 'StringTrim',
    'posted' => 'StringTrim',
    'text' => 'StringTrim',
    'from' => 'StringTrim',
    'to' => 'StringTrim',
  );

  $validators = array();

  $inputFilter = new Zend_Filter_Input($filters,$validators);

  $inputFilter->setData($data);

  if ($inputFilter->isValid()) {
   $params=array("title"=> $inputFilter->title,
     "posted"=> $inputFilter->posted,
     "text"=> $inputFilter->text,
     "from"=> $inputFilter->from,
     "to"=> $inputFilter->to,
   );
  } else {
   //invalid data
   $messages[]="Data are invalid:";
   $messages=array_merge($messages,Utils::getMessagesOneArray($inputFilter));
   $this->view->messages=$messages;
   $this->_forward("searchEntry");
   return;
  }
 }

 //get current page number, default is 1
 $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;
 $itemsPerPage=PageNav::PER_PAGE_COUNT;

 //search object
 $search=Search::factorySearch("SearchBlogArticle");
 //set the pagination
 $search->setPageParams($pageNo,$itemsPerPage);
 
 //set search parameters if these are in the request, otherwise these are loaded from
 //a session in the search factory
 if (isset($params)) {
  $search->setParams($params);
 }
 
 //save search params to the session
 $search->saveToSession();

 //search result
 $article=new Article();
 $this->view->articles=$article->getFound($search);

 //get found items count
 $itemsCount = $search->itemsCount();

 //page navigator
 $pgNav=new PageNav($pageNo,$itemsCount,$itemsPerPage);
 $pgNav->MainUrl=AppUrlEnum::getInstance()->blogSearchEntryResult;

 $this->view->articleCount=$itemsCount;
 $this->view->foundCount=$itemsCount;

 //add page navigator XHTML code for a view
 $this->view->pageNav=$pgNav->getNavXhtmlContent();
 $this->_helper->viewRenderer->setRender('entryList');
}

//==============================================================================
/**
 * Views a dialog for addition of a new article.
 * It is used as target for the forward or redirect from the another actions, hence
 * it can set a form data from an old request, which will not serve in another actions.
 */ 
public function addEntryAction()
{
 $author=new Author();
 $this->view->authors=$author->getAll();

 $category=new Category();
 $this->view->allCategories=$category->getAll();

 //article categories are empty by default
 $articleCategories=array();

 if ($this->_helper->flashMessenger->hasMessages()) {
  $this->view->messages=$this->_helper->flashMessenger->getMessages();
 }

 //in POST are old data, it occures when current submitted data are invalid, so
 //we print old data in form, user can corrects these.
 if (isset($_POST["title"])) {
  $data=$_POST;
  if (isset($data["articleCategories"])) {
   //set article categories from the database by previous submitted category values
   $articleCategories=$category->findByArrId( $data["articleCategories"]);
   unset($data["articleCategories"]);
  }
  $this->view->formData=$data;
 }
 
 $this->view->articleCategories=$articleCategories;
}

//==============================================================================
/**
 * Action, which is as aim for the reciving of the add new article entry dialog data.
 * It tries to save a new article entry into the database, if something fails,
 * forwards to the addEntryAction. If additions was OK, redirects
 * to the addEntryAction with some flash message.
 */ 
public function addToDbEntryAction()
{
 // Don't auto render
 $this->_helper->viewRenderer->setNoRender();
 $data=$_POST;

 $inputFilter=self::getBlogEntryFormFilter();
 $inputFilter->setData($data);

 if ($inputFilter->isValid()) {
  $authorId=$inputFilter->author_id;
  $title=$inputFilter->getUnescaped('title');
  $titleUrl=Utils::friendlyUrl($inputFilter->getUnescaped('title'));
  $postedDate=new Zend_Date($inputFilter->posted);
  $text=$inputFilter->getUnescaped('text');

  $article=new Article();

  try {
   $article->setAuthorId($authorId);
   $article->setTitle($title);
   $article->setTitleUrl($titleUrl);
   $article->setPosted($postedDate->get(Zend_Date::YEAR)."-"
       . $postedDate->get(Zend_Date::MONTH)."-"
       . $postedDate->get(Zend_Date::DAY)." "
       . $postedDate->get(Zend_Date::TIMES));
   $article->setText($text);

   $article->insert();
  
   $article->addToCategoriesbyArrId($data["articleCategories"]);

  } catch (Zend_Db_Exception $e) {
   $message="Cannot save record, database error: ".$e->getMessage();
   $this->view->messages=array($message);
   $this->_forward("addEntry");
   return;
  }

  $urlEnum=AppUrlEnum::getInstance();
  $urlEnum->setBlogEntryId($article->getId());
  $newEntryUrl=$urlEnum->editBlogEntry;
  $message="Record <a href=\"".$newEntryUrl."\">$title</a> saved";
  $this->_helper->flashMessenger->addMessage($message);
  $this->_helper->redirector->setGoto("addEntry");
  $this->_helper->redirector->redirectAndExit();
  return;

 } else {
  //invalid data
  $messages[]="Data are invalid:";
  $messages=array_merge($messages,Utils::getMessagesOneArray($inputFilter));
  $this->view->messages=$messages;
  $this->_forward("addEntry");
  return;
 }
}

//==============================================================================
/**
 * Prepares and returns input filter for the blog entry dialog.
 * @return Zend_Filter_Input
 */
public static function getBlogEntryFormFilter()
{
 $filters = array(
    'title'   => 'StringTrim',
    'posted' => 'StringTrim',
    'text' => 'StringTrim'
 );

 $validators = array(
    'author_id'   => array('NotEmpty','Digits'),
    'title'   => 'NotEmpty',
    'posted' => 'NotEmpty',
    'text' => 'NotEmpty'
 );

 $input = new Zend_Filter_Input($filters, $validators);
 return $input;
}

//==============================================================================
/**
 * Views a dialog for an edit article page with a form.
 * It is used as target for the forward or redirect from the another actions, hence
 * it can set a form data from an old request, which will not serve in another actions.
 */ 
public function editEntryAction()
{
 $article=new Article();
 $category=new Category();
 $author=new Author();

 //in POST are old data, it occures when current submitted data are invalid, so
 //we print old data in form, user can corrects these.
 if (isset($_POST["entryId"])) {
  $data=$_POST;
  if (isset($data["articleCategories"])) {
   //set article categories from the database by previous submitted category values
   $articleCategories=$category->findByArrId($data["articleCategories"]);
   unset($data["articleCategories"]);
  } else {
   $articleCategories=array();
  }
  $this->view->formData=$data;
  $this->view->appUrlEnum->setBlogEntryId($data["entryId"]);
  $this->view->formData["id"]=$data["entryId"];
 } else {
  //no old data, gets data from database
  $article->findById($this->_getParam("itemId"));
  $articleCategories=$article->getCategories();
  $this->view->formData=$article->getDaoRow(false)->toArray();

  //special format for a datetime
  $date=new Zend_Date($article->getPosted(),Zend_Date::ISO_8601);
  $this->view->formData["posted"]=$date->get(Zend_Date::DAY)."-"
   .$date->get(Zend_Date::MONTH)."-"
   .$date->get(Zend_Date::YEAR)." "
   .$date->get(Zend_Date::TIMES);

  $this->view->appUrlEnum->setBlogEntryId($article->getId());
 }

 $this->view->authors=$author->getAll();

 $this->view->allCategories=$category->getAll();

 $this->view->articleCategories=$articleCategories;

 if ($this->_helper->flashMessenger->hasMessages()) {
  $this->view->messages=$this->_helper->flashMessenger->getMessages();
 }
}

//==============================================================================
/**
 * Updates article entry in the database.
 * If something fails, forwards to the editBlogEntry. If the database update was OK, redirects
 * to the editBlogEntry with some flash message.
 */ 
public function updateEntryDbAction()
{
 // Don't auto render
 $this->_helper->viewRenderer->setNoRender();
 $data=$_POST;

 $inputFilter=self::getBlogEntryFormFilter();
 $inputFilter->setData($data);

 if ($inputFilter->isValid()) {
  $authorId=$inputFilter->author_id;
  $title=$inputFilter->getUnescaped('title');
  $titleUrl=Utils::friendlyUrl($inputFilter->getUnescaped('title'));
  $postedDate=new Zend_Date($inputFilter->posted);
  $text=$inputFilter->getUnescaped('text');

  $article=new Article();

  try {
   $article->setId($data['entryId']);
   $article->setAuthorId($authorId);
   $article->setTitle($title);
   $article->setTitleUrl($titleUrl);
   $article->setPosted($postedDate->get(Zend_Date::YEAR)."-"
       . $postedDate->get(Zend_Date::MONTH)."-"
       . $postedDate->get(Zend_Date::DAY)." "
       . $postedDate->get(Zend_Date::TIMES));
   $article->setText($text);

   $article->update();
   $article->updateCategories(isset($data["articleCategories"]) ? $data["articleCategories"] : array() );
  } catch (Zend_Db_Exception $e) {
   $message="Cannot update record, database error: ".$e->getMessage();
   $this->view->messages=array($message);
   $this->_forward("editEntry");
   return;
  }

  $urlEnum=AppUrlEnum::getInstance();
  $urlEnum->setBlogEntryId($article->getId());
  $editEntryUrl=$urlEnum->editBlogEntry;
  $message="Record was updated";
  $this->_helper->flashMessenger->addMessage($message);
  $this->_helper->redirector->gotoUrlAndExit($editEntryUrl,array("prependBase"=>false));
  return;

 } else {
  //invalid data
  $messages[]="Data are invalid:";
  $messages=array_merge($messages,Utils::getMessagesOneArray($inputFilter));
  $this->view->messages=$messages;
  $this->_forward("editEntry");
  return;
 }
}

//==============================================================================
/**
 * Action, which is complete responsible for a removing entry from the database.
 * It views dialog for the confirmation, than if we realy want delete item and
 * serves database removing also.   
 */ 
public function deleteEntryAction()
{
 $article=new Article();
 $itemId=$this->_getParam("itemId");
 $article->findById($itemId);

 //"really want" delete flag
 $delflag=false;

 $urlEnum=AppUrlEnum::getInstance();
 $urlEnum->setBlogEntryId($itemId);

 //confirmation was send
 if (isset($_GET['wantdel']))
 {
  //No - redirect entry list
  if ($_GET['wantdel']=="No") {
   $this->_helper->redirector->gotoUrlAndExit($urlEnum->blogEntryList,array("prependBase"=>false));
   return;
  }
  //Yes - delete
  $delflag = $_GET['wantdel']=="Yes" ? true : false;
 }

 //if really want delete, make it
 if ($delflag) {
  //delete from database
  try {
   $article->delete();
  } catch (Zend_Db_Exception $e) {
   $message="Cannot delete record, database error: ".$e->getMessage();
   $this->_helper->flashMessenger->addMessage($message);
   $this->_helper->redirector->gotoUrlAndExit($urlEnum->deleteBlogEntry,array("prependBase"=>false));
   return;
  }

  //redirect after successful deleting
  $this->_helper->flashMessenger->addMessage("Entry with ID: ".$article->getId()
    . " title: ".$article->getTitle()." was deleted");
  $this->_helper->redirector->gotoUrlAndExit($urlEnum->blogEntryList,array("prependBase"=>false));
  return;
 }
 
 $this->view->subject = "blog entry "
  .$article->getId().": ".$article->getTitle();
 $this->view->wasdeleted=$delflag;
 //delete url
 $this->view->deleteUrl=$urlEnum->deleteBlogEntry;

 if ($this->_helper->flashMessenger->hasMessages()) {
  $this->view->messages=$this->_helper->flashMessenger->getMessages();
 }

 $this->_helper->viewRenderer->setRender("deleteItem");
}

//==============================================================================
/**
 * Views the table with the category list, a pagination is supported.
 */  
public function categoryListAction()
{
 $category = new Category();
 $itemsCount=$category->getCount();
 $this->view->categoryCount=$itemsCount;
 if ($itemsCount) {
  $this->view->categories=$category->getAll();
 }

 if ($this->_helper->flashMessenger->hasMessages()) {
  $this->view->messages=$this->_helper->flashMessenger->getMessages();
 }
}

//==============================================================================
/**
 * Views a dialog for an addition of a new category.
 * It is used as target for the forward or redirect from the another actions, hence
 * it can set a form data from an old request, which will not serve in another actions.
 */ 
public function addCategoryAction()
{
 if ($this->_helper->flashMessenger->hasMessages()) {
  $this->view->messages=$this->_helper->flashMessenger->getMessages();
 }

 //in POST are old data, it occures when current submitted data are invalid, so
 //we print old data in form, user can corrects these.
 if (isset($_POST["name"])) {
  $data=$_POST;
  $this->view->formData=$data;
 }
}

//==============================================================================
/**
 * Action, which is as aim for the reciving of the add new category dialog data.
 * It tries to save a new category into the database, if something fails,
 * forwards to the addEntryAction. If additions was OK, redirects
 * to the addEntryAction with some flash message.
 */ 
public function addToDbCategoryAction()
{
 // Don't auto render
 $this->_helper->viewRenderer->setNoRender();
 $data=$_POST;

 $inputFilter=self::getBlogCategoryFormFilter();
 $inputFilter->setData($data);

 if ($inputFilter->isValid()) {
  $name=$inputFilter->getUnescaped("name");
  $titleUrl=$inputFilter->titleURL;

  $category=new Category();

  try {
   $category->setName($name);
   $category->setTitleUrl($titleUrl);
   $category->insert();
  } catch (Zend_Db_Exception $e) {
   $message="Cannot save record, database error: ".$e->getMessage();
   $this->view->messages=array($message);
   $this->_forward("addCategory");
   return;
  }

  $urlEnum=AppUrlEnum::getInstance();
  $urlEnum->setBlogCategoryId($category->getId());
  $newEntryUrl=$urlEnum->editBlogCategory;
  $message="Record <a href=\"".$newEntryUrl."\">$name</a> saved. Now, you can "
   . "<a href=\"".$urlEnum->editBlogCategoryArticles."\">add</a> some articles into this category.";
  $this->_helper->flashMessenger->addMessage($message);
  $this->_helper->redirector->setGoto("addCategory");
  $this->_helper->redirector->redirectAndExit();
  return;

 } else {
  //invalid data
  $messages[]="Data are invalid:";
  $messages=array_merge($messages,Utils::getMessagesOneArray($inputFilter));
  $this->view->messages=$messages;
  $this->_forward("addCategory");
  return;
 }
}

//==============================================================================
/**
 * Views a dialog for an edit category page with a form.
 * It is used as target for the forward or redirect from the another actions, hence
 * it can set a form data from an old request, which will not serve in another actions.
 */ 
public function editCategoryAction()
{
 $category=new Category();

 //in POST are old data, it occures when current submitted data are invalid, so
 //we print old data in form, user can corrects these.
 if (isset($_POST["categoryId"])) {
  $data=$_POST;
  $this->view->formData=$data;
  $this->view->formData["id"]=$data["categoryId"];
  $category->findById($data["categoryId"]);
 } else {
  //no old data, gets data from database
  $category->findById( $this->_getParam("itemId") );
  $this->view->formData=$category->getDaoRow(false)->toArray();
  $this->view->appUrlEnum->setBlogCategoryId($category->getId());
 }//end isset($_POST["categoryId"]

 $this->view->articleCount=$category->getArticleCount();
 
 if ($this->_helper->flashMessenger->hasMessages()) {
  $this->view->messages=$this->_helper->flashMessenger->getMessages();
 }
}

//==============================================================================
/**
 * Updates category in the database.
 * If something fails, forwards to the editCategoryAction. If the database update  was OK, redirects
 * to the editCategoryAction with some flash message.
 */ 
public function updateCategoryDbAction()
{
 // Don't auto render
 $this->_helper->viewRenderer->setNoRender();
 $data=$_POST;

 $inputFilter=$this->getBlogCategoryFormFilter();
 $inputFilter->setData($data);

 if ($inputFilter->isValid()) {
  $name=$inputFilter->name;
  $titleUrl=$inputFilter->titleURL;

  $category=new Category();

  try {
   $category->setId($data["categoryId"]);
   $category->setName($name);
   $category->setTitleUrl($titleUrl);
   $category->update();
  } catch (Zend_Db_Exception $e) {
   $message="Cannot update record, database error: ".$e->getMessage();
   $this->view->messages=array($message);
   $this->_forward("editCategory");
   return;
  }

  $urlEnum=AppUrlEnum::getInstance();
  $urlEnum->setBlogCategoryId($data["categoryId"]);
  $message="Record was updated";
  $this->_helper->flashMessenger->addMessage($message);
  $this->_helper->redirector->gotoUrlAndExit($urlEnum->editBlogCategory,array("prependBase"=>false));
  return;

 } else {
  //invalid data
  $messages[]="Data are invalid:";
  $messages=array_merge($messages,Utils::getMessagesOneArray($inputFilter));
  $this->view->messages=$messages;
  $this->_forward("editCategory");
  return;
 }
}

//==============================================================================
/**
 * Views a dialog for a form, which is responsible for the adding and removing
 * articles from one particular category. It also server database operation, when form
 * is sended.  
 */ 
public function editCategoryArticlesAction()
{
 $category=new Category();

 $category->findById( $this->_getParam("itemId") );
 $this->view->category=$category;
 $this->view->appUrlEnum->setBlogCategoryId($category->getId());
 
 //data was send
 if (isset($_POST["action"])) {
  $data=$_POST;
  try {
   $category->updateArticles( isset($data["assigned"]) ? $data["assigned"] : array() );
  } catch (Zend_Db_Exception $e) {
   unset($_POST["action"]);
   $message="Cannot update record, database error: ".$e->getMessage();
   $this->view->messages=array($message);
   $this->_forward("editCategoryArticles");
   return;
  }

  $urlEnum=$this->view->appUrlEnum;
  $message="Articles for this category were updated.";
  $this->_helper->flashMessenger->addMessage($message);
  $this->_helper->redirector->gotoUrlAndExit($urlEnum->editBlogCategoryArticles,array("prependBase"=>false));
  return;
 }

 $this->view->categoryArticles=$category->getArticles();

 $article=new Article();
 $this->view->allArticles=$article->getAll();
 
 if ($this->_helper->flashMessenger->hasMessages()) {
  $this->view->messages=$this->_helper->flashMessenger->getMessages();
 }
}

//==============================================================================
/**
 * Action, which is complete responsible for a removing category from the database.
 * It views dialog for the confirmation, than if we realy want delete item and
 * serves database removing also.   
 */ 
public function deleteCategoryAction()
{
 $category=new Category();
 $itemId=$this->_getParam("itemId");
 $currentCategory=$category->findById($itemId);

 //"really want" delete flag
 $delflag=false;

 $urlEnum=AppUrlEnum::getInstance();
 $urlEnum->setBlogCategoryId($itemId);

 //confirmation was send
 if (isset($_GET['wantdel']))
 {
  //No - redirect to category list
  if ($_GET['wantdel']=="No") {
   $this->_helper->redirector->gotoUrlAndExit($urlEnum->blogCategoryList,array("prependBase"=>false));
   return;
  }
  //Yes - delete
  $delflag = $_GET['wantdel']=="Yes" ? true : false;
 }

 //if really want delete, make it
 if ($delflag) {
  //delete from database
  try {
   $category->delete();
  } catch (Zend_Db_Exception $e) {
   $message="Cannot delete record, database error: ".$e->getMessage();
   $this->_helper->flashMessenger->addMessage($message);
   $this->_helper->redirector->gotoUrlAndExit($urlEnum->deleteBlogCategory,array("prependBase"=>false));
   return;
  }

  //redirect after successful deleting
  $this->_helper->flashMessenger->addMessage("Category with ID: ".$category->getId()
    . " name: ".$category->getName()." was deleted");
  $this->_helper->redirector->gotoUrlAndExit($urlEnum->blogCategoryList,array("prependBase"=>false));
  return;
 }
 
 $this->view->subject = "blog category ".$category->getId()." ".$category->getName();
 $this->view->nextInfo = "Be aware, when you delete a category, all atricles and their comments will be deleted also.";

 $this->view->wasdeleted=$delflag;
 //delete url
 $this->view->deleteUrl=$urlEnum->deleteBlogCategory;

 if ($this->_helper->flashMessenger->hasMessages()) {
  $this->view->messages=$this->_helper->flashMessenger->getMessages();
 }

 $this->_helper->viewRenderer->setRender("deleteItem");
}

//==============================================================================
/**
 * Prepares and returns input filter for the category entry dialog.
 * @return Zend_Filter_Input
 */
public static function getBlogCategoryFormFilter()
{
 $filters = array(
    'name'   => 'StringTrim',
    'titleURL' => 'StringTrim'
 );

 $validators = array(
    'name'   => 'NotEmpty',
    'titleURL'   => array('NotEmpty',new Zend_Validate_Regex("/\w+/"))
 );

 $input = new Zend_Filter_Input($filters, $validators);
 return $input;
}

//==============================================================================
/**
 * Views the table with the comment list, a pagination is supported.
 */  
public function commentListAction()
{
 $comment = new Comment();
 $itemsCount=$comment->getCount();
 
 $this->view->commentCount=$itemsCount;
 
 if ($itemsCount) {
  //get current page number, default is 1
  $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;
  //page navigator
  $pgNav=new PageNav($pageNo,$itemsCount);
  $pgNav->MainUrl=AppUrlEnum::getInstance()->blogCommentList;

  $this->view->comments=$comment->getAll($pageNo,$pgNav->getPerPageItems());
  
  //add page navigator XHTML code for a view
  $this->view->pageNav=$pgNav->getNavXhtmlContent();
 }

 if ($this->_helper->flashMessenger->hasMessages()) {
  $this->view->messages=$this->_helper->flashMessenger->getMessages();
 }
}

//==============================================================================
/**
 * Views a dialog for an edit comment page with a form.
 * It is used as target for the forward or redirect from the another actions, hence
 * it can set a form data from an old request, which will not serve in another actions.
 */ 
public function editCommentAction()
{
 $comment=new Comment();

 //in POST are old data, it occures when current submitted data are invalid, so
 //we print old data in form, user can corrects these.
 if (isset($_POST["commentId"])) {
  $data=$this->getRequest()->getPost();
  $this->view->formData=$data;
  $this->view->formData["id"]=$data["commentId"];
  $comment->findById( $data["commentId"] );
 } else {
  //no old data, gets data from database
  $comment->findById( $this->_getParam("itemId") );
  $this->view->formData=$comment->getDaoRow(false)->toArray();
  
  //special format for a datetime
  $date=new Zend_Date($comment->getPosted(),Zend_Date::ISO_8601);
  $this->view->formData["posted"]=$date->get(Zend_Date::DAY)."-"
   .$date->get(Zend_Date::MONTH)."-"
   .$date->get(Zend_Date::YEAR)." "
   .$date->get(Zend_Date::TIMES);
  
  $this->view->appUrlEnum->setBlogCommentId($comment->getId());
 }//end isset($_POST["commentId"]

 $relatedArticle=$comment->getArticle();
 $this->view->article=$relatedArticle;
 $urlEnum=AppUrlEnum::getInstance();
 $urlEnum->setBlogEntryName($relatedArticle->getTitleUrl());
 $this->view->articleUrl=$urlEnum->blogEntry;
  
 if ($this->_helper->flashMessenger->hasMessages()) {
  $this->view->messages=$this->_helper->flashMessenger->getMessages();
 }
}

//==============================================================================
/**
 * Search for a article entry - view for a search form
 */ 
public function searchCommentAction()
{
}

//==============================================================================
/**
 * Action, which is aim for a search comment entry dialog.
 * When input data are invalid, forwards to the {@link searchCommentAction()}. 
 */ 
public function searchCommentResultAction()
{
 if ($this->_hasParam("name")) {
  $data=$this->getRequest()->getPost();
  $filters = array(
    'name'   => 'StringTrim',
    'email' => 'StringTrim',
    'message' => 'StringTrim',
    'from' => 'StringTrim',
    'to' => 'StringTrim',
  );

  $validators = array();

  $inputFilter = new Zend_Filter_Input($filters,$validators);

  $inputFilter->setData($data);

  if ($inputFilter->isValid()) {
   $params=array("name"=> $inputFilter->name,
     "email"=> $inputFilter->email,
     "message"=> $inputFilter->message,
     "from"=> $inputFilter->from,
     "to"=> $inputFilter->to,
   );
  } else {
   //invalid data
   $messages[]="Data are invalid:";
   $messages=array_merge($messages,Utils::getMessagesOneArray($inputFilter));
   $this->view->messages=$messages;
   $this->_forward("searchComment");
   return;
  }
 }

 //get current page number, default is 1
 $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;
 $itemsPerPage=PageNav::PER_PAGE_COUNT;

 //search object
 $search=Search::factorySearch("SearchBlogComment");
 //set the pagination
 $search->setPageParams($pageNo,$itemsPerPage);
 
 //set search parameters if are in the requestm, otherwise these are loaded from
 //session in the factory
 if (isset($params)) {
  $search->setParams($params);
 }
 
 //search result
 $comment=new Comment();
 $this->view->comments=$comment->getFound($search);

 //get found items count
 $itemsCount = $search->itemsCount();

 //save search params to session
 $search->saveToSession();

 //page navigator
 $pgNav=new PageNav($pageNo,$itemsCount,$itemsPerPage);
 $pgNav->MainUrl=AppUrlEnum::getInstance()->searchBlogCommentResult;

 $this->view->commentCount=$itemsCount;
 $this->view->foundCount=$itemsCount;
 
 //add page navigator XHTML code for a view
 $this->view->pageNav=$pgNav->getNavXhtmlContent();
 $this->_helper->viewRenderer->setRender('commentList');
}

//==============================================================================
/**
 * Updates the comment entry in the database.
 * If something fails, forwards to the editCommentAction. If the database update was OK, redirects
 * to the editCommentAction with some flash message.
 */ 
public function updateCommentDbAction()
{
 // Don't auto render
 $this->_helper->viewRenderer->setNoRender();
 $data=$this->getRequest()->getPost();

 $filters = array(
    'name' =>'StringTrim',
    'email' => 'StringTrim',
    'www' => 'StringTrim',
    'posted' => 'StringTrim',
    'message' => array('StringTrim',
       new Zend_Filter_StripTags(array("p","br","a"),array("href"))
     )
   );

 $validators = array(
    'name'   => 'NotEmpty',
    'www' => array(new UrlValid(),'allowEmpty' => true), 
    'email'   => 'EmailAddress',
    'message' => 'NotEmpty',
    'posted' => 'NotEmpty',
  );

 $inputFilter = new Zend_Filter_Input($filters, $validators);
 $inputFilter->setData($data);

 if ($inputFilter->isValid()) {
  $name=$inputFilter->getUnescaped("name");
  $email=$inputFilter->getUnescaped("email");
  $message=$inputFilter->getUnescaped("message");
  $postedDate=new Zend_Date($inputFilter->posted); 
  $www=$inputFilter->getUnescaped('www');
  if (!empty($www) && !preg_match("|^http(s)?://|i",$www)) $www="http://".$www; 

  $comment=new Comment();

  try {
   $comment->setId($data["commentId"]);
   $comment->setAuthorName($name);
   $comment->setAuthorEmail($email);
   $comment->setAuthorWWW($www);
   $comment->setPosted($postedDate->get(Zend_Date::YEAR)."-"
    . $postedDate->get(Zend_Date::MONTH)."-"
    . $postedDate->get(Zend_Date::DAY)." "
    . $postedDate->get(Zend_Date::TIMES)); 
   $comment->setMessage($message);
   $comment->update();
  } catch (Zend_Db_Exception $e) {
   $message="Cannot update record, database error: ".$e->getMessage();
   $this->view->messages=array($message);
   $this->_forward("editComment");
   return;
  }

  $urlEnum=AppUrlEnum::getInstance();
  $urlEnum->setBlogCommentId($data["commentId"]);
  $message="Record was updated";
  $this->_helper->flashMessenger->addMessage($message);
  $this->_helper->redirector->gotoUrlAndExit($urlEnum->editBlogComment,array("prependBase"=>false));
  return;

 } else {
  //invalid data
  $messages[]="Data are invalid:";
  $messages=array_merge($messages,Utils::getMessagesOneArray($inputFilter));
  $this->view->messages=$messages;
  $this->_forward("editComment");
  return;
 }
}

//==============================================================================
/**
 * Action, which is complete responsible for a removing comment from the database.
 * It views dialog for the confirmation, than if we realy want delete item and
 * serves database removing also.   
 */ 
public function deleteCommentAction()
{
 $comment=new Comment();
 $itemId=$this->_getParam("itemId");
 $currentComment=$comment->findById($itemId);

 //"really want" delete flag
 $delflag=false;

 $urlEnum=AppUrlEnum::getInstance();
 $urlEnum->setBlogCommentId($itemId);

 //confirmation was send
 if (isset($_GET['wantdel']))
 {
  //No - redirect entry list
  if ($_GET['wantdel']=="No") {
   $this->_helper->redirector->gotoUrlAndExit($urlEnum->blogCommentList,array("prependBase"=>false));
   return;
  }
  //Yes - delete
  $delflag = $_GET['wantdel']=="Yes" ? true : false;
 }

 //if really want delete, make it
 if ($delflag) {
  //delete from database
  try {
   $comment->delete();
  } catch (Zend_Db_Exception $e) {
   $message="Cannot delete record, database error: ".$e->getMessage();
   $this->_helper->flashMessenger->addMessage($message);
   $this->_helper->redirector->gotoUrlAndExit($urlEnum->deleteBlogComment,array("prependBase"=>false));
   return;
  }

  //redirect after successful deleting
  $this->_helper->flashMessenger->addMessage("Comment with ID ".$comment->getId()
   . " was deleted");
  $this->_helper->redirector->gotoUrlAndExit($urlEnum->blogCommentList,array("prependBase"=>false));
  return;
 }

 $this->view->subject = "comment with ID: ".$comment->getId();
 $this->view->nextInfo="<br />Comment text: ".$comment->getMessage();
 $this->view->wasdeleted=$delflag;
 //delete url
 $this->view->deleteUrl=$urlEnum->deleteBlogComment;

 if ($this->_helper->flashMessenger->hasMessages()) {
  $this->view->messages=$this->_helper->flashMessenger->getMessages();
 }

 $this->_helper->viewRenderer->setRender("deleteItem");
}

#END CLASS
}
