<?php
/**
 * Implementation of blog Index controller actions
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Serve blog index actions
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class Blog_IndexController extends Zend_Controller_Action
{
//==============================================================================
/**
 * Adds some paths for view scripts and setups right block on the page
 */   
public function init()
{
 $config = Zend_Registry::get("conf");
 $this->view->addHelperPath($config->defaultViewDir.DIRECTORY_SEPARATOR."helpers");
 $this->view->addScriptPath($config->defaultViewDir.DIRECTORY_SEPARATOR."scripts");

 //right block
 $category=new Category();
 $this->view->allCategories=$category->getAll(null,null,array("article"));
 $this->view->rightBlock=$this->view->render("blogMenu.phtml");
 //unset, because $this->view is used in controller actions and must be without any
 //variables in these actions
 unset($this->view->allCategories);
}

//==============================================================================
/**
 * Action handler for index page.
 */
public function indexAction()
{
 if ($this->_hasParam("entry")) {
  $this->_forward("viewEntry");
  return;
 }

 $article = new Article();
 
 //get current page number, default is 1
 $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;

 //page navigator
 $pgNav=new PageNav($pageNo,$article->getCount(),Article::ARTICLES_PERPAGE);
 $pgNav->MainUrl=AppUrlEnum::getInstance()->blogIndex;

 $this->view->articles=$article->getAll($pageNo,$pgNav->getPerPageItems(),
   array("comment","author")
 );

 //add page navigator XHTML code for a view
 $this->view->pageNav=$pgNav->getNavXhtmlContent();
 
 //page title
 $this->view->pageTitle="Blog";
}

//==============================================================================
/**
 * Action handler for one blog entry.
 * Serves add action for comments and view comments for the current article also.   
 */
public function viewEntryAction()
{
 $articleTitleUrl=$this->_getParam("entry");
 $currentArticle = new Article();
 
 $found=$currentArticle->findByTitleUrl($articleTitleUrl);
 //false - article not found
 if (!$found) {
  if (HomeWebApp::DEBUG) {
   echo "article doesn't exist, in production, 404 is invoked";
   exit;
  } else { 
   $this->_forward("error","Error",null,
    array("error_handler"=>Zend_Controller_Plugin_ErrorHandler::EXCEPTION_NO_ACTION));
   return;
  } 
 }
 
 $this->view->article=$currentArticle;
 $this->view->author=$currentArticle->getAuthor();
 $this->view->comments=$currentArticle->getComments();

 //page title
 $this->view->pageTitle=$currentArticle->getTitle();

 $urlEnum=AppUrlEnum::getInstance();
 $urlEnum->setBlogEntryName($currentArticle->getTitleUrl());
 $currentEntryUrl=$urlEnum->blogEntry;

 //just alias for $this->_helper->flashMessenger
 $messanger=$this->_helper->getHelper('FlashMessenger');

 //vaditate and save comment if it's submitted via form
 if ($this->_hasParam("formAction") && $this->_hasParam("formAction")=="addCommentDb") {
  $data=$this->getRequest()->getPost();

  $filters = array(
    'username' =>'StringTrim',
    'mainEmail' => 'StringTrim',
    'message' => 'StringTrim',
    'www' => array('StringTrim')
   );

  $validators = array(
    'username'   => 'NotEmpty',
    'mainEmail'   => 'EmailAddress',
    'message' => 'NotEmpty',
    'www' => array(new UrlValid(),'allowEmpty' => true),
    'homepage' => array(
        'Alnum',
        'presence' => 'required'
     ) 
  );

  $inputFilter = new Zend_Filter_Input($filters, $validators);
  $inputFilter->setData($data);

  //filter is valid, we can save comment
  if ($inputFilter->isValid()) {
   //check spam phrase
   if ($inputFilter->homepage!="NejsemSpammer") {
    $messanger->addMessage("Spam not allowed");
    $this->_helper->redirector->setGotoUrl($currentEntryUrl,array("prependBase"=>false));
    $this->_helper->redirector->redirectAndExit();
    return;
   }

   $username=htmlspecialchars($inputFilter->getUnescaped("username"));
   $email=$inputFilter->getUnescaped('mainEmail');
   $commentMessage=$inputFilter->getUnescaped('message');
   $commentMessage=Comment::prepareCommentText($commentMessage);

   $www=$inputFilter->getUnescaped('www');
   if (!empty($www) && !preg_match("|^http(s)?://|i",$www)) $www="http://".$www;
     
   try {
    $comment=new Comment();
    $comment->setArticleId($currentArticle->getId());
    $comment->setMessage($commentMessage);
    $comment->setAuthorName($username);
    $comment->setAuthorEmail($email);
    $comment->setAuthorWWW($www);

    $comment->insert();
   } catch (Zend_Db_Exception $e) {
    $messages[]="Sorry, cannot save record comment, database error was occured"
     . (HomeWebApp::DEBUG ? ":".$e->getMessage() : ".");
    $this->view->messages=$messages;
    //remove formAction parameter from the $_POST, it's impossible unset superglobal via zend request class
    unset($_POST["formAction"]);
    $this->_forward("viewEntry");
    return;
   }
   //some OK message, actually not necessary
  } else {
   //invalid data
   $messages[]="Data are invalid:";
   $messages=array_merge($messages,Utils::getMessagesOneArray($inputFilter));
   $this->view->messages=$messages;
   //remove formAction parameter from the $_POST, it's impossible unset superglobal
   //via zend request class
   unset($_POST["formAction"]);
   $this->_forward("viewEntry");
   return;
  }

  $this->_helper->redirector->setGotoUrl($currentEntryUrl,array("prependBase"=>false));
  $this->_helper->redirector->redirectAndExit();
 } //end if ($this->_hasParam("formAction") ...

 if ($messanger->hasMessages()) {
  $this->view->messages=$messanger->getMessages();
 }

 if ($this->_hasParam("username")) {
  $this->view->formData=$this->getRequest()->getPost();
 }
 
}

//==============================================================================
/**
 * Action hadler for a view of articles category.
 */
public function viewCategoryAction()
{
 $category=new Category();
 $category->findByTitleUrl($this->_getParam("name"));
 $this->view->category=$category;

 //page title
 $this->view->pageTitle=$this->view->trans("Articles in the category")." "
  .$this->view->escape($category->getName());

 $urlEnum=AppUrlEnum::getInstance();
 $urlEnum->setBlogCategoryName($category->getTitleUrl());

 $allArticleCount=$category->getArticleCount();

 //get current page number, default is 1
 $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;

 //page navigator
 $pgNav=new PageNav($pageNo,$allArticleCount,Article::ARTICLES_PERPAGE);
 $pgNav->MainUrl=$urlEnum->blogCategory;

 //category articles in the current page
 $this->view->articles=$category->getArticles($pageNo,$pgNav->getPerPageItems());

 $this->view->allArticleCount=$allArticleCount;
 
 //add page navigator XHTML code for a view
 $this->view->pageNav=$pgNav->getNavXhtmlContent();
}

//==============================================================================
/**
 * Builds and prints RSS feed for some count if recent articles.
 * Sets "isFeed" parameter, which says to controller plugin that this
 * response isn't part of the normal site template.
 */
public function feedAction()
{
 // Don't auto render
 $this->_helper->viewRenderer->setNoRender();
 $article = new Article();
 $recentArticles=$article->getAll(1,10);

 $lastUpdate=current($recentArticles)->getPosted();
 $urlEnum=AppUrlEnum::getInstance();

 $baseUri=AppBaseUrl::getUri();

 //prepare an array for a feed
 $feedArray = array(
  "title" => "Mirin's blog",
	"link" => $baseUri.$urlEnum->blogRSSfeed,
	"lastUpdate" => strtotime($lastUpdate),
  "charset" => "utf-8",
	"description" => "Blog about coding, and other stuff",
	"author" => "Mirek Kubelik",
	"email" => "koubel@volny.cz",
	"copyright" => "Koubel, all rights reserved",
  "generator" => 'Zend Framework Zend_Feed',
	"language" => "cs",
			'entries' => array()
		);

 foreach ($recentArticles as $article) {
  $urlEnum->setBlogEntryName($article->getTitleUrl());
  $feedArray['entries'][] = array(
   "title" => iconv("iso-8859-2","utf-8",$article->getTitle()),
	 "link" => $baseUri.$urlEnum->blogEntry,
	 "description" => iconv("iso-8859-2","utf-8",
      strip_tags(Article::getFirstParagraph( $article->getText() ))),
	 "lastUpdate" => strtotime($article->getPosted()),
	 "content" => iconv("iso-8859-2","utf-8",Article::escapeCode($article->getText()))
	 );
 }
 
 $feed = Zend_Feed::importArray($feedArray, 'rss');
 $feed->send();

 //response is feed
 $this->_setParam("isFeed",true);
}

//==============================================================================
/**
 * Builds and print RSS feed for some recent comments.
 * Sets "isFeed" parameter, which says to controller plugin that this
 * response isn't part of the normal template.
 */
public function commentFeedAction()
{
 // Don't auto render
 $this->_helper->viewRenderer->setNoRender();
 $comment=new Comment();
 //last 20 recent comments
 $recentComments=$comment->getAll(1,20);

 $lastUpdate=current($recentComments)->getPosted();
 $urlEnum=AppUrlEnum::getInstance();

 $baseUri=AppBaseUrl::getUri();

 //prepare an array for a feed
 $feedArray = array(
  "title" => "Mirin's blog - recent comments",
	"link" => $baseUri.$urlEnum->blogRSScommentFeed,
	"lastUpdate" => strtotime($lastUpdate),
  "charset" => "utf-8",
	"description" => "recent comments in Mirin's blog",
	"author" => "Mirek Kubelik",
	"email" => "koubel@volny.cz",
	"copyright" => "Koubel, all rights reserved",
  "generator" => 'Zend Framework Zend_Feed',
	"language" => "cs",
			'entries' => array()
		);

 foreach ($recentComments as $comment) {
  $relatedArticle=$comment->getArticle();
  //set article params in the url enumerator
  $urlEnum->setBlogEntryName($relatedArticle->getTitleUrl());
  //comment order in all article comments
  $order=$comment->getOrderforArticle();
  $message=iconv("iso-8859-2","utf-8",Comment::commentBody( $comment->getMessage() ));

  $feedArray['entries'][] = array(
   "title" => iconv("iso-8859-2","utf-8",$comment->getAuthorName()
     . " ("
     . $relatedArticle->getTitle()
     . ")"
    ),
	 "link" => $baseUri.$urlEnum->blogEntry."#comment$order",
	 "description" => $message,
	 "lastUpdate" => strtotime($comment->getPosted()),
	 "content" => $message
	 );
 }
 
 $feed = Zend_Feed::importArray($feedArray, 'rss');
 $feed->send();

 //response is feed
 $this->_setParam("isFeed",true);
}

//==============================================================================
/**
 * Views result from the main search form, it searches for an atricles
 * via simple regexp for article body. 
 */
public function searchResultAction()
{
 //get current page number, default is 1
 $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;
 
 //search object
 $search=Search::factorySearch("SearchBlogArticle");
 //set the pagination
 $search->setPageParams($pageNo,Article::ARTICLES_PERPAGE);
 
 $params = Utils::getHTTPParamsFiltered(array("submit"));
 
 //set search parameters if form is submitted, otherwise these are set in the
 //factory method
 if (isset($params["searchText"]))
  $search->setParams($params);

 $article=new Article();

 //found items
 $this->view->foundItems=$article->getFound($search);

 //found items count
 $itemsCount = $search->itemsCount();

 //save search params to session
 $search->saveToSession();

 //page navigator
 $pgNav=new PageNav($pageNo,$itemsCount,Article::ARTICLES_PERPAGE);
 $pgNav->MainUrl=AppUrlEnum::getInstance()->blogSearchResult;

 $this->view->allFoundItemsCount=$itemsCount;
 
 //add page navigator XHTML code for a view
 $this->view->pageNav=$pgNav->getNavXhtmlContent();
}

//==============================================================================
/**
 * Actions for a view of articles for one month.   
 */
public function viewArchiveAction()
{
 $year=$this->_getParam("year");
 $month=$this->_getParam("month");
 
 $app=HomeWebApp::getInstance();

 $date=new Zend_Date(array('year' => $year,
    'month' => $month,
    'day'=>1),$app->getLocale());

 $this->view->month=iconv("utf-8","iso-8859-2",Utils::getMonthFirstCaseName($month,$app->getLocale()));
 $this->view->year=$date->get(Zend_Date::YEAR);
 //page title
 $this->view->pageTitle=$this->view->trans("Articles in")." ".$this->view->month
  ." ".$this->view->year;
 
 $urlEnum=AppUrlEnum::getInstance();
 $urlEnum->setBlogArchiveYear($year);
 $urlEnum->setBlogArchiveMonth($month);

 //get current page number, default is 1
 $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;

 $article=new Article();
 //all articles for the current archive
 $allArticleCount=$article->getMonthItemsCount($year,$month);

 $this->view->allArticleCount=$allArticleCount;

 //page navigator
 $pgNav=new PageNav($pageNo,$allArticleCount,Article::ARTICLES_PERPAGE);
 $pgNav->MainUrl=$urlEnum->blogArchive;

 //articles in the current page for current archive year and month
 $this->view->articles=$article->getMonthItems($year,$month,$pageNo,$pgNav->getPerPageItems());

 //add page navigator XHTML code for a view
 $this->view->pageNav=$pgNav->getNavXhtmlContent();
}

#END CLASS
}
