<?php
/**
 * Article model.
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Article model class  
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class Article extends Model
{
 /** articles per page */
 const ARTICLES_PERPAGE = 5;

 /** identifier
  * @var int
  */  
 private $_id;

 /** title for an article
  * @var string
  */  
 private $_title;
 
 /** "niced" url identifier which is derived format of the title
  * @var string
  */  
 private $_titleUrl;

 /** article text
  * @var string
  */  
 private $_text; 

 /** creation datetime for the article 
  * @var string the format is mostly Zend_Date::ISO_8601
  */  
 private $_posted;

 /** author identifier 
  * @var int
  */  
 private $_authorId;

 /** count of comment for the article 
  * @var int
  */  
 private $_commentCount;

 /** article author name 
  * @var string
  */  
 private $_authorName;


//==============================================================================
/**
 * @param $dao if is null, use {@link Dao_Article} as default Dao.
 * @return Article
 */  
public function __construct(Dao $dao=null)
{
 $usedDao=is_null($dao) ?  new Dao_Article() : $dao;
 parent::__construct($usedDao);
}

//==============================================================================
public function getId()
{
 return $this->_id;
}

//==============================================================================
public function setId($id)
{
 $this->_id=$id;
}

//==============================================================================
public function getTitle()
{
 return $this->_title;
}

//==============================================================================
public function setTitle($title)
{
 $this->_title=$title;
}

//==============================================================================
public function getTitleUrl()
{
 return $this->_titleUrl;
}

//==============================================================================
public function setTitleUrl($titleUrl)
{
 $this->_titleUrl=$titleUrl;
}

//==============================================================================
public function getText()
{
 return $this->_text;
}

//==============================================================================
public function setText($text)
{
 $this->_text=$text;
}

//==============================================================================
public function getPosted()
{
 return $this->_posted;
}

//==============================================================================
public function setPosted($posted)
{
 $this->_posted=$posted;
}

//==============================================================================
public function getAuthorId()
{
 return $this->_authorId;
}

//==============================================================================
public function setAuthorId($id)
{
 $this->_authorId=$id;
}

//==============================================================================
public function getCommentCount()
{
 if (!isset($this->_commentCount)) {
  $comment=new Comment();
  $this->_commentCount=$comment->getCountByArticle($this);
 }
 return $this->_commentCount;
}

//==============================================================================
public function getAuthorName()
{
 if (!isset($this->_authorName)) {
  $this->getAuthor();
 }
 return $this->_authorName;
}

//==============================================================================
public function getAuthor()
{
 $author=new Author();
 $author->setByArticle($this);
 
 $this->_authorName=$author->getUsername();

 return $author;
}

//==============================================================================
public function setByComment(Comment $comment)
{
 $currentComment=$comment->getDaoRow();
 $articleRow=$currentComment->findParentRow("Dao_Article");
 $this->_setFromDaoRow($articleRow->toArray());
}

//==============================================================================
public function getAll($pageNo=null,$perPageItems=null,$extendedInfo=null)
{
 $rows=$this->getDao()->fetchAllItems($pageNo,$perPageItems,$extendedInfo);

 return $this->_getSet($rows);
}

//==============================================================================
protected function _setPropFromArray(&$arr)
{
 foreach($arr as $key=>$val) {
  $this->{"_".$key}=$val;
 }
}

//==============================================================================
public function findByTitleUrl($titleUrl)
{
 $row=$this->getDao()->findByTitleUrl($titleUrl);

 if (is_null($row)) {
  return false;
 } else {
  $this->_setFromDaoRow($row->toArray());
  return true;
 }
}

//==============================================================================
public function findById($id)
{
 $row=$this->getDao()->find($id)->current();
 $this->_setFromDaoRow($row->toArray());
}

//==============================================================================
public function insert()
{
 $this->_id=$this->getDao()->insert(
     array("title"=>$this->_title,
      "titleURL"=>$this->_titleUrl,
      "author_id"=>$this->_authorId,
      "posted"=>$this->_posted,
      "text"=>$this->_text,
     )
 );
}

//==============================================================================
public function update()
{
 $this->getDao()->update(
   array(
    "author_id"=>$this->_authorId,
    "title"=>$this->_title,
    "titleUrl"=>$this->_titleUrl,
    "posted"=>$this->_posted,
    "text"=>$this->_text
   ), "id=$this->_id"
 );
}

//==============================================================================
public function delete()
{
 $this->getDao()->delete("id=$this->_id");
}

//==============================================================================
public function addToCategoriesbyArrId($categoriesArrId)
{
 $categoryArticleDao=new Dao_CategoryArticle();
 foreach($categoriesArrId as $categoryId) {
  $categoryArticleDao->insert(array("article_id"=>$this->_id,"category_id"=>$categoryId));
 }
}

//==============================================================================
public function updateCategories($newCategoriesIds)
{
 $currentArticleRow=$this->getDaoRow();

 //get current article categories IDs
 $currentArticleCategories=$currentArticleRow->findManyToManyRowset('Dao_Category', 'Dao_CategoryArticle');
 $currentIds=array();
 foreach($currentArticleCategories as $category) {
  $currentIds[]=$category->id;
 }

 $categoryArticleDao=new Dao_CategoryArticle();

 //new categories
 $diff=array_diff($newCategoriesIds,$currentIds);
 foreach($diff as $catId) {
  $categoryArticleDao->insert(
   array("category_id"=>$catId,"article_id"=>$this->_id)
  );
 }

 //old caregories - categories which were assigned and now aren't, these
 //will be removed
 $diff=array_diff($currentIds,$newCategoriesIds);
 foreach($diff as $catId) {
  $categoryArticleDao->delete("category_id=$catId AND article_id=$this->_id");
 }
}

//==============================================================================
/**
 * Gets database row object which corresponds our current properties.
 * It is used mostly for the operation with the relationship. Hence we need
 * to set only keys
 * @param bool $keyOnly true - sets only key properties (primary and foreign) it is
 * enough in most situations, false - sets all properties
 * @return Zend_Db_Table_Row_Abstract
 */
public function getDaoRow($keyOnly=true)
{
 $data = array(
  "id"=>$this->_id,
  "author_id"=>$this->_authorId
 );
 
 if (!$keyOnly) {
  $data["title"]=$this->_title;
  $data["titleURL"]=$this->_titleUrl;
  $data["posted"]=$this->_posted;
  $data["text"]=$this->_text;
 }

 $row=$this->getDao()->createRow($data);
 return $row;
}

//==============================================================================
public function getCategories()
{
 $category=new Category();
 return $category->getByArticle($this);
}

//==============================================================================
/**
 * Gets articles in the specific category.
 *
 * findManyToManyRowset method cannot be used, because it doesn't support a pagination
 *
 * @return array array of Article objects
 */
public function getByCategory(Category $category,$pageNo=null,$perPageItems=null)
{
 $categoryArticlesDao=new Dao_CategoryArticle();
 $catoryArticlesRows=$categoryArticlesDao->getArticlesByCategory($category->getId(),$pageNo,$perPageItems);

 return $this->_getSet($catoryArticlesRows);
}

//==============================================================================
public function getComments()
{
 $comment=new Comment();
 return $comment->getByArticle($this);
}

//==============================================================================
/**
 * Gets count of articles for the specific month in the specific year.
 * Just proxy for the dao method call.
 * @param int $year
 * @param int $month
 * @returns int
 */
public function getMonthItemsCount($year,$month)
{
 return $this->getDao()->getMonthItemsCount($year,$month);
}

//==============================================================================
/**
 * Returns article list for the specific month in the specific year.
 * Just proxy for the dao method call. A pagination is supported.
 * @param int $year
 * @param int $month
 * @param int $pageNo current page number, firts page has no. 1
 * @param int $perPageItems items per page
 * @return array result table as array
 */
public function getMonthItems($year,$month,$pageNo=null,$perPageItems=null)
{
 $rows=$this->getDao()->getMonthItems($year,$month,$pageNo,$perPageItems);
 
 return $this->_getSet($rows);
}

//==============================================================================
protected function _inflectDaoRowIndexes(&$arr)
{
 $notAllowedIndexes=array("dao");

 $keys=array_keys($arr);
 foreach($keys as $key) {
  //unset not allowed indexes
  if (in_array($key,$notAllowedIndexes))
   throw new Exception("you cannot use $key index as the column index");

  switch ($key) {
   case "author_id":
    Utils::renameKey($arr,$key,"authorId");
    break;
   case "titleURL":
    Utils::renameKey($arr,$key,"titleUrl");
    break;
   default:
    break;
  }
 }
}

//==============================================================================
/**
 * Returns first paragraph from the article html text.
 * If none find returns whole article.
 * @param sting $articleText article text as the html
 * @return string
 */
public static function getFirstParagraph($articleText)
{
 preg_match("/<p>.*<\/p>/sU",$articleText,$matches);
 return empty($matches) ? $articleText : $matches[0] ;
}

//==============================================================================
/**
 * Makes html special chars conversion for all text in the <code> tag for a whole
 * article text.
 * @param string $articleText article text as the html
 * @return string
 */
public static function escapeCode($articleText)
{
 $pattern = "/<code>((\\n|.)*)<\/code>/U";
 $text=preg_replace_callback($pattern,array('Article', 'escapeCodeCallback'),$articleText);
 return $text;

 //alernative, but with non-transparent $replacement, stripslashes in a replacement
 //needed because /e regexp modifier slashes all ' "
 //$pattern = "/<code>((\\n|.)*)<\/code>/eU";
 //$replacement="'<code>'.htmlspecialchars(stripslashes('\\1'),ENT_NOQUOTES).'</code>'";
 //$text=preg_replace($pattern,$replacement,$articleText);
}

//==============================================================================
/**
 * Callback for the {@link escapeCode()}
 * @param array $matches array with regexp matches
 * @return string
 */
public static function escapeCodeCallback($matches)
{
 return "<code>".htmlspecialchars($matches[1])."</code>";
}

#END CLASS
}
