<?php
/**
 * Dao - simple extension for the Zend_Db_Table.
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * This class add some usefull functionality onto Zend_Db_Table
 * class, e.g row count.
 * 
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
abstract class Dao extends Zend_Db_Table_Abstract
{
 /**
  * data source for the table
  * @var DataSource
  */
 protected static $_dataSource;

//==============================================================================
/**
 * Sets data source for and default adapter for the
 * table. 
 */ 
public function __construct()
{
 self::$_dataSource=DataSource::getInstance();
 if (is_null(self::getDefaultAdapter())) {
  self::setDefaultAdapter(self::$_dataSource->getAdapter());
 
  $config = Zend_Registry::get("conf");
    
  //cache all tables metadata
  $frontendOptions = array(
    'automatic_serialization' => true
   );
  $backendOptions  = array(
    'cacheDir' => $config->cacheDir
   );
  $cache = Zend_Cache::factory('Core', 'File', $frontendOptions, $backendOptions);
  Zend_Db_Table_Abstract::setDefaultMetadataCache($cache);
 }

 parent::__construct();
}

//==============================================================================
/**
 * Data source accessor.
 * @return DataSource 
 */ 
public function getDataSource()
{
 return self::$_dataSource;
}

//==============================================================================
/**
 * Gets row count in the table.
 * @return int
 */
public function getCount()
{
 $sql="select count(*) from $this->_name";
 return $this->getAdapter()->fetchOne($sql);
}

#END CLASS
}
