<?php
/**
 * Common blog database searching functionality. 
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Common blog database searching functionality. 
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
abstract class Search_Blog extends Search
{
//==============================================================================
/**
 * Builds a SQL search command.
 * @param bool $withPages use LIMIT for pagination
 * @param bool $countOnly SQL for count of rows will be returned
 * @return Zend_Db_Select
 */
protected function _buildSQL($withPages,$countOnly=false)
{
  $select=DataSource::getInstance()->getAdapter()->select();
  
  //columns and from parts
  $select->from(implode(",",$this->_fromTables),
   $countOnly ? $this->_buildCountSql() : "*"
  );

  //where part
  $this->_sqlWhereCond($select);

  //add group by aggregation columns
  //don't add group by if we make distint count select
  if (!empty($this->_groupByColums) && !$countOnly) {
   $select->group($this->_groupByColums);
  }

  //add statements for ordering
  $this->_buildOrderSQL($select);

  //add limit if necessary
  if ($withPages && !$countOnly)
   $select->limitPage($this->_pageNumber,$this->_perPageItems);

  //self::log("SQL for search: $sql");

  return $select;
}

//==============================================================================
/**
 * Builds string for the count function.
 * @return string
 */
private function _buildCountSQL()
{
 $countParams = empty($this->_countDistinct) ? "(*)" : "(distinct ".implode(",",$this->_countDistinct).")";
 return "count".$countParams;
}

//==============================================================================
/**
 * Builds string for the ORDER statement in the SQL command.
 * @return string
 */
protected function _buildOrderSQL(Zend_Db_Select $select)
{
 throw new RuntimeException("_buildOrderSQL() function isn't implemented");
}

//==============================================================================
/**
 * Builds string for a condition after WHERE in SQL command.
 * Conditions are in the array from the {@link _getSqlCondition()} 
 * @return string
 */
abstract protected function _sqlWhereCond(Zend_Db_Select $select);

//==============================================================================
/**
 * Returns set of rows, which are matched by "code" conditions. 
 * 
 * It is used in
 * that case where we cannot use the condition via SQL database. Match criteria 
 * are specified via {@link _rowMatch()} method.
 *  
 * @return array searched rows
 */
protected function _selectRowsByCode()
{
 if ($this->_paging) {
  $beginIndex=($this->_pageNumber-1)*$this->_perPageItems+1;
  $endIndex=$beginIndex+$this->_perPageItems;
 }
 
 $select = $this->_buildSQL(false);
   
 $stmt = $select->query();
 $searchedItems=array();

 $this->_foundCount=0;
 
 while ($row = $stmt->fetch()) {
  if ($this->_rowMatch($row)) {
   $this->_foundCount++;
   if ($this->_paging) { 
    if ($this->_foundCount>=$beginIndex && $this->_foundCount<$endIndex) {
     $searchedItems[]=$row;
    }
    continue;
   } else {
    $searchedItems[]=$row;
   }
  }
 } //end while

 $stmt->close();

 return $searchedItems;
}

//==============================================================================
protected function _rowMatch($row)
{
 throw new RuntimeException("_rowMatch function isn't implemented");
}

#END CLASS
}
